@@ -19,8 +19,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-from __future__ import print_function
-
 import errno
 import os
 import signal
@@ -73,6 +71,7 @@ class Install(install_misc.InstallBase):
 
         self.select_language_packs(save=True)
         self.select_ecryptfs()
+        self.keep_ad_tools()
         if self.db.get('ubiquity/install/generate-blacklist') == 'true':
             self.db.progress('START', 0, 100, 'ubiquity/install/title')
             self.db.progress('INFO', 'ubiquity/install/blacklist')
@@ -246,6 +245,14 @@ class Install(install_misc.InstallBase):
         else:
             difference = set()
 
+        # Add minimal installation package list if selected
+        if self.db.get('ubiquity/minimal_install') == 'true':
+            if os.path.exists(install_misc.minimal_install_rlist_path):
+                pkgs = set()
+                with open(install_misc.minimal_install_rlist_path) as m_file:
+                    pkgs = {line.strip().split(':')[0] for line in m_file}
+                difference |= pkgs
+
         cache = Cache()
 
         use_restricted = True
@@ -257,7 +264,8 @@ class Install(install_misc.InstallBase):
         if not use_restricted:
             for pkg in cache.keys():
                 if (cache[pkg].is_installed and
-                        cache[pkg].section.startswith('restricted/')):
+                        cache[pkg].candidate.section.startswith(
+                            'restricted/')):
                     difference.add(pkg)
 
         # Keep packages we explicitly installed.
@@ -268,15 +276,20 @@ class Install(install_misc.InstallBase):
         # at file copy time, we should figure out why grub still fails when
         # apt-install-direct is present during configure_bootloader (code
         # removed).
-        if arch in ('amd64', 'i386'):
+        if arch in ('amd64', 'arm64', 'i386'):
+            # We now want grub-pc to be left installed for both EFI and legacy
+            # since it makes sure that the right maintainer scripts are run.
+            keep.add('grub-pc')
             if subarch == 'efi':
                 keep.add('grub-efi')
                 keep.add('grub-efi-amd64')
+                keep.add('grub-efi-arm64')
                 keep.add('grub-efi-amd64-signed')
+                keep.add('grub-efi-arm64-signed')
+                keep.add('flash-kernel')
+                keep.add('aarch64-laptops-support')
                 keep.add('shim-signed')
                 keep.add('mokutil')
-                keep.add('fwupdate-signed')
-                install_misc.record_installed(['fwupdate-signed'])
                 try:
                     altmeta = self.db.get(
                         'base-installer/kernel/altmeta')
@@ -287,14 +300,6 @@ class Install(install_misc.InstallBase):
                 keep.add('linux-signed-generic%s' % altmeta)
             else:
                 keep.add('grub')
-                keep.add('grub-pc')
-        elif (arch in ('armel', 'armhf') and
-              subarch in ('omap', 'omap4', 'mx5')):
-            keep.add('flash-kernel')
-            keep.add('u-boot-tools')
-        elif arch == 'powerpc':
-            keep.add('yaboot')
-            keep.add('hfsutils')
 
         # Even adding ubiquity as a depends to oem-config-{gtk,kde} doesn't
         # appear to force ubiquity and libdebian-installer4 to copy all of
@@ -430,6 +435,7 @@ class Install(install_misc.InstallBase):
 
                 # Is the path blacklisted?
                 if (not stat.S_ISDIR(st.st_mode) and
+                        '/' in relpath and
                         '/%s' % relpath in self.blacklist):
                     if debug:
                         syslog.syslog('Not copying %s' % relpath)
@@ -745,6 +751,14 @@ class Install(install_misc.InstallBase):
                     install_misc.record_installed(['ecryptfs-utils'])
                     break
 
+    def keep_ad_tools(self):
+        """Mark Active Directory tools to be kept for now
+
+        The decision in plugininstall (after the copy starts) will tell us
+        if the user has selected or not AD integration, and so, which tools to remove."""
+
+        install_misc.record_installed(['adcli', 'krb5-config', 'realmd', 'sssd'])
+
 
 if __name__ == '__main__':
     os.environ['DPKG_UNTRANSLATED_MESSAGES'] = '1'
