@@ -24,12 +24,6 @@
 import string
 import syslog
 
-if __name__ == "__main__":
-    # This is done by kde_ui.py. We need to do the same for our test main(),
-    # but it must be done *before* importing any PyQt5 module
-    import sip
-    sip.setapi('QVariant', 1)
-
 from PyQt5 import QtCore
 from PyQt5 import QtGui, QtWidgets
 
@@ -272,7 +266,13 @@ class NetworkManagerTreeView(QtWidgets.QTreeView):
 
     def connect_to_selection(self, passphrase):
         devid, ssid = self._get_selected_row_ids()
-        self.wifi_model.connect_to_ap(devid, ssid, passphrase)
+        try:
+            self.wifi_model.connect_to_ap(devid, ssid, passphrase)
+        except Exception as e:
+            dialog = QtWidgets.QMessageBox()
+            dialog.setWindowTitle("Failed to connect to wireless network")
+            dialog.setText("{}".format(e))
+            dialog.exec_()
 
     def get_cached_passphrase(self):
         index = self.currentIndex()
